import { Clipboard } from "clipeasy";
var template = "\nclass SimpleDsu {\n  protected _set!: number[];\n\n  public get dataSet() {\n    return this._set;\n  }\n\n  constructor(n: number) {\n    this.init(n);\n  }\n\n  /**\n   * \u521D\u59CB\u5316\u5E76\u67E5\u96C6\n   * @param n\n   */\n  public init(n: number) {\n    this._set = Array.from({\n      length: n,\n    }).fill(-1) as number[];\n  }\n\n  /**\n   * \u67E5\u627Enum\u5728\u96C6\u5408\u4E2D\u662F\u5426\u5B58\u5728\uFF0C\u82E5\u5B58\u5728\u8FD4\u56DE\u5176\u6240\u5728\u96C6\u5408\u7684\u8DDF\u8282\u70B9\u7684\u7D22\u5F15\uFF08\u5373\u7F16\u53F7\uFF09\uFF0C\u82E5\u4E0D\u5B58\u5728\uFF0C\u5219\u8FD4\u56DE-1\n   * @param num\n   * @returns\n   */\n  public find(num: number) {\n    let idx = num - 1;\n    // \u67E5\u627E\u5E76\u67E5\u96C6\u4E2D\u4E0D\u5B58\u5728\u7684\u5143\u7D20\n    if (idx < 0 || idx >= this._set.length) {\n      return -1;\n    }\n    // \u5411\u4E0A\u8FED\u4EE3\u5BFB\u627E\u7236\u5143\u7D20\n    while (this._set[idx] >= 0) {\n      idx = this._set[idx];\n    }\n    return idx;\n  }\n\n  /**\n   * \u5408\u5E76\u4E24\u4E2A\u96C6\u5408\uFF0C\u6309\u79E9\u5F52\u5E76\n   * @param num1\n   * @param num2\n   */\n  public union(num1: number, num2: number) {\n    let idx1 = this.find(num1);\n    let idx2 = this.find(num2);\n    // \u8BF4\u660E\u5927\u5BB6\u4E0D\u5728\u540C\u4E00\u4E2A\u96C6\u5408\u5185\n    if (idx1 != idx2) {\n      // idx1\u7684\u79E9\u5C0F\u4E8Eidx2\u7684\u79E9\n      if (this._set[idx1] > this._set[idx2]) {\n        // \u5C06idx1\u7684\u8DDF\u8282\u70B9\u6307\u5411idx2\u7684\u6839\u8282\u70B9\n        this._set[idx1] = idx2;\n      } else {\n        if (this._set[idx1] === this._set[idx2]) {\n          this._set[idx1]--;\n        }\n        this._set[idx2] = idx1;\n      }\n    }\n  }\n\n  /**\n   * \u7EDF\u8BA1\u96C6\u5408\u7684\u4E2A\u6570\n   * @returns\n   */\n  public count() {\n    return this._set.reduce((total, item) => {\n      return total + (item < 0 ? 1 : 0);\n    }, 0);\n  }\n}\n";
/**
 * 简单版本的并查集实现，即数组的索引即代表的是它所存储的数据
 */
var SimpleDsu = /** @class */ (function () {
    function SimpleDsu(n) {
        this.init(n);
        new Clipboard().write(template);
    }
    Object.defineProperty(SimpleDsu.prototype, "dataSet", {
        get: function () {
            return this._set;
        },
        enumerable: false,
        configurable: true
    });
    /**
     * 初始化并查集
     * @param n
     */
    SimpleDsu.prototype.init = function (n) {
        this._set = Array.from({
            length: n,
        }).fill(-1);
    };
    /**
     * 查找num在集合中是否存在，若存在返回其所在集合的跟节点的索引（即编号），若不存在，则返回-1
     * @param num
     * @returns
     */
    SimpleDsu.prototype.find = function (num) {
        var idx = num - 1;
        // 查找并查集中不存在的元素
        if (idx < 0 || idx >= this._set.length) {
            return -1;
        }
        // 向上迭代寻找父元素
        while (this._set[idx] >= 0) {
            idx = this._set[idx];
        }
        return idx;
    };
    /**
     * 合并两个集合，按秩归并
     * @param num1
     * @param num2
     */
    SimpleDsu.prototype.union = function (num1, num2) {
        var idx1 = this.find(num1);
        var idx2 = this.find(num2);
        // 说明大家不在同一个集合内
        if (idx1 != idx2) {
            // idx1的秩小于idx2的秩
            if (this._set[idx1] > this._set[idx2]) {
                // 将idx1的跟节点指向idx2的根节点
                this._set[idx1] = idx2;
            }
            else {
                if (this._set[idx1] === this._set[idx2]) {
                    this._set[idx1]--;
                }
                this._set[idx2] = idx1;
            }
        }
    };
    /**
     * 统计集合的个数
     * @returns
     */
    SimpleDsu.prototype.count = function () {
        return this._set.reduce(function (total, item) {
            return total + (item < 0 ? 1 : 0);
        }, 0);
    };
    return SimpleDsu;
}());
export { SimpleDsu };
